/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License("CDDL") (the "License").  You may not use this file
 * except in compliance with the License. 
 * 
 * You can obtain a copy of the License at:
 *     https://jersey.dev.java.net/license.txt
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * When distributing the Covered Code, include this CDDL Header Notice in each
 * file and include the License file at:
 *     https://jersey.dev.java.net/license.txt
 * If applicable, add the following below this CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 *     "Portions Copyrighted [year] [name of copyright owner]"
 */

package com.sun.ws.rest.api.core;

import java.util.Set;

/**
 * Methods describing the set of resources in a Web application. It is anticipated
 * that an implementation of this interface will be generated by tools rather
 * than directly implemented by developers.
 */
public interface ResourceConfig {
    /**
     * Get the set of resource classes that should be deployed. A resource class
     * is a class with a <code>UriTemplate</code> annotation.
     * @return the set of resource classes.
     */
    Set<Class> getResourceClasses();

    /**
     * Specifies whether matrix parameters are ignored for the purposes
     * of matching a request URI to a resource class.
     * @return true if matrix params should be ignored, false if not
     */
    boolean isIgnoreMatrixParams();

    /**
     * Specifies whether the runtime should silently normalize the request URI
     * or issue a client redirect to the normailzed request URI.
     * @return true if redirects will be used, false if not
     */
    boolean isRedirectToNormalizedURI();
}
