/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License("CDDL") (the "License").  You may not use this file
 * except in compliance with the License. 
 * 
 * You can obtain a copy of the License at:
 *     https://jersey.dev.java.net/license.txt
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * When distributing the Covered Code, include this CDDL Header Notice in each
 * file and include the License file at:
 *     https://jersey.dev.java.net/license.txt
 * If applicable, add the following below this CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 *     "Portions Copyrighted [year] [name of copyright owner]"
 */

package com.sun.ws.rest.impl.resolver;

import com.sun.ws.rest.api.container.ContainerException;
import com.sun.ws.rest.api.core.HttpRequestContext;
import com.sun.ws.rest.spi.resolver.WebResourceResolver;
import com.sun.ws.rest.spi.resolver.WebResourceResolverListener;

/**
 *
 * @author Paul.Sandoz@Sun.Com
 */
public final class StatelessResolver implements WebResourceResolver {
    private Class resourceClass;
    private Object resource;

    public StatelessResolver(Class resourceClass) {
        this.resourceClass = resourceClass;
    }

    public Class<?> getWebResourceClass() {
        return resource.getClass();
    }
    
    public Object resolve(HttpRequestContext request, WebResourceResolverListener listener) {
        if (resource == null) {
            instantiate();
            listener.onInstantiation(resource);
        }
        return resource;
    }
    
    private void instantiate() {
        try {
            resource = resourceClass.newInstance();
        } catch (IllegalAccessException e) {
            throw new ContainerException("Illegal access to Class " + resourceClass, e);
        } catch (InstantiationException e) {
            throw new ContainerException("Instantiation error for Class " + resourceClass, e);
        }
    }
    
}
